;;; -*- mode: emacs-lisp -*-
;;; Copyright (C) 2002 by Alexei Meremianin
;;;
;;; Gnu Emacs interface for Mova Dictionaries. Version 0.1 ;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  Tested with Mueller english-russian dictionary
;;;  ver.3.4, (C) E.S.Cymbalyuk 1999. Under GNU GPL. No Warranty. 
;;;  The latest versions see on 
;;;  http://www.chat.ru/~mueller_dic or
;;;  http://www.geocities.com/mueller_dic 
;;;
;;; Features:
;;;  - Searches for a word under cursor and using minibuffer's prompt
;;;  - Highlights dictionary article
;;;
;;; Requirements:
;;;  - A dictionary file in `mova' format and Unix `grep' command
;;;
;;; Limitaions:
;;;  - Nice syntax highlighting works only for English language
;;;  - Can't display phonetic symbols
;;; 
;;; Bugs: 
;;;  - Don't know
;;;
;;; Installation:
;;;  - Copy this file into your load-path
;;;  - Optionally byte-compile it for speed
;;;  - Add to your ~/.emacs following strings:
;;;    (require 'elmova)
;;;    (global-set-key [(f3)] 'elmova) ; choose your own
;;;                                    ; key instead of F3
;;;  - Customize group `elmova' for faces, paths etc.
;;;
;;; Tested with GNU Emacs 21.1, and Mueller Dictionary ver. 3.4
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                  Copyright.
;;; GNU Public License. No warranty of any kind. Use at your own risk.
;;;
;;;       <hbar02@yahoo.de>
;;;   Unhappy Xemacs users need to search somewhere for `emova.el' 
;;;  written by Pavel Kurnosoff <pqr@yasp.com>
;;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;

(defgroup elmova nil
  "Search in dictionaries in 'Mueller' format"
  :group 'help)

(defcustom elmova-dictionary "/home/alexei/share/dict/Mueller24.koi" "\
*Path to dictionary."
  :type 'string
  :group 'elmova)

;; (defvar elmova-dictionary-hash-file (concat elmova-dictionary ".hash") "\
;; *Name of a hash file if any.")

(defface elmova-shorten-face 
  '((((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (((class color) (background light)) (:foreground "blue" :italic t))
    (((class color) (background dark)) (:foreground "blue" :italic t)))
  "Dictionary shortening face"
  :group 'elmova)

(defface elmova-word-face 
  '((((class grayscale) (background light)) (:foreground "DimGray"))
    (((class grayscale) (background dark)) (:foreground "LightGray"))
    (((class color) (background light)) (:foreground "ForestGreen"))
    (((class color) (background dark)) (:foreground "ForestGreen")))
  "Dictionary word face"
  :group 'elmova)

(defface elmova-underline-face 
  '((((class grayscale) (background light)) (:foreground "DimGray" :underline t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :underline t))
    (((class color) (background light)) (:foreground "blue" :underline t))
    (((class color) (background dark)) (:foreground "blue" :underline t)))
  "Dictionary underline face"
  :group 'elmova)
;;;;;;
;;;
;;; Main proceduce, which do grep, format and colors.
;;;
(defun elmova ()
  (interactive)
  (save-selected-window
  (defvar elmova-word nil)  ;; word to translate
  (defvar point-temp)   ;; where the point is in the basic text
  (setq point-temp (point))
  (if (< point-temp (buffer-size)) (forward-char 1))
  (re-search-backward "[A-Za-z]" nil t)
  (if (thing-at-point 'word) 
      (setq elmova-word (read-string
			 (concat "English word (default `"
				 (thing-at-point 'word) "'): ")
				     nil nil (thing-at-point 'word)))
    (setq elmova-word (read-string "Enter english word: "))
    )
  (setq elmova-word (downcase elmova-word))
;;  (goto-char (1+ (match-beginning 0)))
  (goto-char point-temp)
;;;;;;;;
;;;;;;;; Create buffer *dictionary*  ;;;;;;;;;
;;;;;;;;
    (get-buffer-create "*dictionary*")  
    (switch-to-buffer-other-window "*dictionary*")
;    (erase-buffer)
    (goto-char (point-max))
    (setq point-temp (point-max)) ;; now point-temp shows the end 
;; of an old *dictionary* buffer

;; grep'ping in dictionary file
 (call-process "grep" nil "*dictionary*" nil "-e" 
       (concat "^" elmova-word " ") elmova-dictionary)


;;;; Don't show the *distionary* buffer when the translation is absent
 (if (= point-temp (point-max)) (progn 
			   (princ "Sorry. No matches found." t)
			   (delete-windows-on "*dictionary*"))
  (progn
;;;; control the width of text 
    (setq fill-column 78)
;;;;;;;;
;; Break the line at ">" and "0-9." and "_I.."
    (goto-char (point-max))
    (insert "\n  ---------------------------------\n")
    (while (re-search-backward "\\s-.+>\\|\\s-[0-9]+\\.\\|_I"
			       point-temp t)
      (insert "\n"))
;;;;
;;;;     filling and justifying
;;;;
;; 1. Level
    (setq left-margin 0)
    (goto-char point-temp)
    (while (re-search-forward (concat "^" elmova-word ".+$") nil t)
      (fill-region (match-beginning 0) (match-end 0) nil))
    (setq left-margin 2)
    (goto-char point-temp)
    (while (re-search-forward "[0-9]+\\..+$" nil t)
      (fill-region-as-paragraph (match-beginning 0) (match-end 0)))

;; 1> Level
    (goto-char point-temp)
    (setq left-margin 4)
    (while (re-search-forward "[0-9]>.+$" nil t)
      (fill-region-as-paragraph (match-beginning 0) (match-end 0)))

;;;;  a> b> Level
    (goto-char point-temp)
    (setq left-margin 8)
    (while (re-search-forward "[^0-9]>.+$" nil t)
      (fill-region-as-paragraph (match-beginning 0) (match-end 0)))

;;;; Changing size of the window
;;     (if (< (count-lines point-temp (point-max)) (window-height))
;; 	    (set-window-text-height (count-lines point-temp (point-max))))
;    (shrink-window-if-larger-than-buffer)
    (setq window-min-height 4)

;; Do some nice colorizing..
;;; searching english word
    (goto-char point-temp)
    (while (re-search-forward (concat "[a-z (),-/']*" 
				      elmova-word "[ a-z(),-/']*[^a-z]") nil t)
      (put-text-property (match-beginning 0) 
			 (1- (match-end 0)) 
			 'face 'elmova-word-face))
    (goto-char (point-max))
    (while (re-search-backward "_\\([^ ]+\\)" point-temp t)
      (replace-match "\\1" nil nil)
      (put-text-property (match-beginning 0) (1- (match-end 0)) 
			 'face 'elmova-shorten-face))
    (goto-char point-temp)
    (while (re-search-forward (concat elmova-word "[a-z]*") nil t)
      (put-text-property (match-beginning 0) (match-end 0)
			 'face 'elmova-underline-face))
    (goto-char (point-max))
    (while (re-search-backward " [0-9]+>" point-temp t)
      (delete-char -1))
    (goto-char point-temp)
    (set-window-start
           (selected-window)
           point-temp)
    )
  (princ "Use M-C-v to scroll the translation" t)
 ))
)
;;;;;

(provide 'elmova) 






